"""
Python wrapper for `diamond.sh`
"""
from os import path
import os
import subprocess
import database


def run(device, source, no_trce=False, mapargs=None, backanno=False):
    """
    Run diamond.sh with a given device name and source Verilog file
    """
    env = os.environ.copy()
    if no_trce:
        env["NO_TRCE"] = "1"
    if backanno:
        env["BACKANNO"] = "1"
    if mapargs is not None:
        env["MAPARGS"] = mapargs
    dsh_path = path.join(database.get_trellis_root(), "diamond.sh")
    return subprocess.run(["bash",dsh_path,device,source], env=env)




"""
Utilities for fuzzing cell timing
"""
from fuzzconfig import FuzzConfig
import fuzzloops
import cell_timings
import timing_dbs
from os import path


def timing_configs(job, design, density="45"):
    # speedgrade, fuzzconfig
    return [
        ("6", FuzzConfig(job + "_6", "ECP5", "LFE5U-{}F".format(density), [], design, "-s 6")),
        ("7", FuzzConfig(job + "_7", "ECP5", "LFE5U-{}F".format(density), [], design, "-s 7")),
        ("8", FuzzConfig(job + "_8", "ECP5", "LFE5U-{}F".format(density), [], design, "-s 8")),
        ("8_5G", FuzzConfig(job + "_8_5G", "ECP5", "LFE5UM5G-{}F".format(density), [], design, "-s 8"))
    ]


def build_and_add(designs, density="45", inc_cell=cell_timings.include_cell, rw_cell_func=cell_timings.rewrite_celltype,
                  rw_pin_func=cell_timings.rewrite_pin):
    jobs = []
    sdfs = dict()
    for des in designs:
        jobs += timing_configs(path.basename(des).replace(".v", ""), des, density)
    for job in jobs:
        grade, cfg = job
        sdfs[grade] = []

    def per_job(job):
        grade, cfg = job
        cfg.setup(skip_specimen=True)
        bitf = cfg.build_design(cfg.ncl, {}, backanno=True, substitute=False)
        sdf = bitf.replace(".bit", ".sdf")
        sdfs[grade].append(sdf)

    fuzzloops.parallel_foreach(jobs, per_job)
    for grade in sdfs.keys():
        db = timing_dbs.cells_db_path("ECP5", grade)
        for sdf in sdfs[grade]:
            cell_timings.add_sdf_to_database(db, sdf, include_cell_predicate=inc_cell, rewrite_cell_func=rw_cell_func,
                                             rewrite_pin_func=rw_pin_func)



"""
Database and Database Path Management
"""
import os
from os import path
import json
import subprocess


def get_trellis_root():
    """Return the absolute path to the Project Trellis repo root"""
    return path.abspath(path.join(__file__, "../../../"))


def get_db_root():
    """
    Return the path containing the Project Trellis database
    This is database/ in the repo, unless the `PRJTRELLIS_DB` environment
    variable is set to another value.
    """
    if "PRJTRELLIS_DB" in os.environ and os.environ["PRJTRELLIS_DB"] != "":
        return os.environ["PRJTRELLIS_DB"]
    else:
        return path.join(get_trellis_root(), "database")


def get_db_subdir(family = None, device = None, package = None):
    """
    Return the DB subdirectory corresponding to a family, device and
    package (all if applicable), creating it if it doesn't already
    exist.
    """
    subdir = get_db_root()
    dparts = [family, device, package]
    for dpart in dparts:
        if dpart is None:
            break
        subdir = path.join(subdir, dpart)
        if not path.exists(subdir):
            os.mkdir(subdir)
    return subdir


def get_tilegrid(family, device):
    """
    Return the deserialised tilegrid for a family, device
    """
    tgjson = path.join(get_db_subdir(family, device), "tilegrid.json")
    with open(tgjson, "r") as f:
        return json.load(f)


def get_devices():
    """
    Return the deserialised content of devices.json
    """
    djson = path.join(get_db_root(), "devices.json")
    with open(djson, "r") as f:
        return json.load(f)


def get_db_commit():
    return subprocess.getoutput('git -C "{}" rev-parse HEAD'.format(get_db_root()))



import sys
import json
import timing_dbs
import pip_classes


def make_interconn_timing_html(dbfile, family, grade, htmlfile):
    with open(dbfile, 'r') as f:
        db = json.load(f)
    with open(htmlfile, 'w') as html:
        print("<html>", file=html)
        print("<head>", file=html)
        print("<title>{} Speed Grade -{} Interconnect Timings</title>".format(family, grade), file=html)
        print("</head>", file=html)
        print("<body>", file=html)
        print("<h1>{} Speed Grade -{} Interconnect Timings</h1>".format(family, grade), file=html)
        print("<p>Values in red are fixed to zero by definition.</p>", file=html)
        print("<table width='800'>", file=html)
        print("<tbody>", file=html)
        print("<tr><th rowspan='2'>Pip Class</th>", file=html)
        print("<th colspan='3'>Base Delay (ps)</th><th colspan='3'>Fanout Adder (ps)</th></tr>", file=html)
        print("<tr><th>Min</th><th>Typ</th><th>Max</th><th>Min</th><th>Typ</th><th>Max</th></tr>", file=html)

        trstyle = ""
        for pipclass, pcdata in sorted(db.items()):
            trstyle = " bgcolor=\"#dddddd\"" if trstyle == "" else ""
            print("<tr {}>".format(trstyle), file=html)
            print("<td>{}</td>".format(pipclass), file=html)
            ds = "style='color: red;'" if pip_classes.force_zero_delay_pip(pipclass) else ""
            fs = "style='color: red;'" if pip_classes.force_zero_fanout_pip(pipclass) else ""
            print("<td {}>{:.0f}</td><td {}>{:.0f}</td><td {}>{:.0f}</td>".format(ds, pcdata["delay"][0], ds, pcdata["delay"][1], ds, pcdata["delay"][2]), file=html)
            print("<td {}>{:.0f}</td><td {}>{:.0f}</td><td {}>{:.0f}</td>".format(fs, pcdata["fanout"][0], fs, pcdata["fanout"][1], fs, pcdata["fanout"][2]), file=html)
            print("</tr>", file=html)
        print("</tbody>", file=html)
        print("</table>", file=html)
        print("</body>", file=html)
        print("</html>", file=html)


def main(argv):
    if len(argv) < 3:
        print("./interconnect_html.py grade out.html")
    make_cell_timing_html(timing_dbs.interconnect_db_path("ECP5", argv[1]), "ECP5", argv[1], argv[2])


if __name__ == "__main__":
    sys.exit(main(sys.argv))
